#!/bin/csh -f
#
# build-apps.csh: build the S2PLOT bundled applications 
#
# David Barnes, November 2007
#
# $Id: build-apps.csh 5670 2011-09-30 05:34:38Z dbarnes $

if (!(${?S2PATH})) then
  echo "S2PATH environment variable MUST be set ... please fix and retry."
  exit(-1);
endif
if (! -d $S2PATH || ! -e ${S2PATH}/s2plot.csh) then
  echo "S2PATH is set but invalid: ${S2PATH} ... please fix and retry."
  exit(-1);
endif
source ${S2PATH}/s2plot.csh
if ($status) then
  exit(-1)
endif

#set thisdir=$cwd
set thisdir=`pwd`

if ($thisdir != $S2PATH) then
  echo "You must be in directory ${S2PATH} to run build-samples.csh script."
  echo "(or if you are, try removing any trailing '/' in your S2PATH setting)"
  exit(-1);
endif

if ($S2SHARED == yes) then
  echo "Using shared S2PLOT library for ${S2KERNEL}."
else
  echo "Using static S2PLOT library for ${S2KERNEL}."
endif

if (! -d $S2KERNEL) then
  echo "Directory ${S2KERNEL} does not exist!  Cannot build!  Try running"
  echo "build-lib.csh script first, or re-installing your distribution."
  exit(-1);
endif

if (! -e ${S2KERNEL}/${S2LIBNAME}) then
  echo "Library ${S2LIBNAME} is missing!  Cannot build!  Try running "
  echo "build-lib.csh script first, or re-installing your distribution."
  exit(-1);
endif

echo "Entering target directory ${S2KERNEL}."
cd $S2KERNEL

set S2OBJECTS=""

echo "Compiling s2slides application ..."
$S2CCMPILER ../apps/s2slides/s2slides.cc -I../apps/s2slides 
set S2OBJECTS="${S2OBJECTS} s2slides.o"
echo "Linking s2slides application ..."
#echo $S2CCINKER -o s2slides s2slides.o -L${S2PATH}/${S2KERNEL} ${S2LINKS} ${MLLINKS} ${SWLINKS} ${GLLINKS} -L${S2X11PATH}/lib${S2LBITS} ${S2FORMSLINK} -lXpm -lX11 ${IMATH} -lm ${XLINKPATH} -lpthread
$S2CCINKER -o s2slides s2slides.o -L${S2PATH}/${S2KERNEL} ${S2LINKS} ${MLLINKS} ${SWLINKS} ${GLLINKS} -L${S2X11PATH}/lib${S2LBITS} ${S2FORMSLINK} -lX11 ${IMATH} -lm ${XLINKPATH} -lpthread

if ($S2KERNEL == darwin && $S2SHARED == yes) then
  echo "Modifying dynamic library path ..."
  install_name_tool -change ${S2PATH}/${S2KERNEL}/${S2LIBNAME} @executable_path/${S2LIBNAME} s2slides
endif

echo "Compiling s2anim application ..."
$S2COMPILER ../apps/s2anim/s2anim.c -I../apps/s2anim 
set S2OBJECTS="${S2OBJECTS} s2anim.o"
echo "Linking s2anim application ..."
$S2CCINKER -o s2anim s2anim.o -L${S2PATH}/${S2KERNEL} ${S2LINKS} ${MLLINKS} ${SWLINKS} ${GLLINKS} -L${S2X11PATH}/lib${S2LBITS} ${S2FORMSLINK} -lX11 ${IMATH} -lm ${XLINKPATH} -lpthread

if ($S2KERNEL == darwin && $S2SHARED == yes) then
  echo "Modifying dynamic library path ..."
  install_name_tool -change ${S2PATH}/${S2KERNEL}/${S2LIBNAME} @executable_path/${S2LIBNAME} s2anim
endif

echo "Compiling xrw2pdf application ..."
$S2COMPILER ../apps/xrw2pdf/xrw2pdf.c -I../apps/xrw2pdf
set S2OBJECTS="${S2OBJECTS} xrw2pdf.o"
echo "Linking xrw2pdf application ..."
$S2CCINKER -o xrw2pdf xrw2pdf.o -L${S2PATH}/${S2KERNEL} ${S2LINKS} ${MLLINKS} ${SWLINKS} ${GLLINKS} -L${S2X11PATH}/lib${S2LBITS} ${S2FORMSLINK} -lX11 ${IMATH} -lm ${XLINKPATH} -lpthread -lz

if ($S2KERNEL == darwin && $S2SHARED == yes) then
  echo "Modifying dynamic library path ..."
  install_name_tool -change ${S2PATH}/${S2KERNEL}/${S2LIBNAME} @executable_path/${S2LIBNAME} xrw2pdf
endif


echo Removing objects ...
rm -rf $S2OBJECTS

echo Done!
