/* s2plot.js: Adobe 3D JavaScript for S2PLOT-exported 3-d content
 *
 * Use of this script implies acceptance of the requirement that at
 * least one of the following papers be cited when the script is used
 * to produce a published work:
 * 
 * 1. Barnes, D.G., Fluke, C.J., 2008, "Incorporating interactive
 *    3-dimensional graphics in astronomy research papers", New
 *    Astronomy, 13, 599.
 *
 * 2. Fluke, C.J., Barnes, D.G., 2008, "The Interactive Astronomy
 *    Textbook, Astronomy Education Review", 7(1).
 *
 * ==================================================================
 * S2PLOT is freely available for non-commercial and educational use,
 * however, it is not public-domain software. The programming library
 * and documentation may not be redistributed or sub-licensed in any
 * form without permission from Swinburne University of
 * Technology. The programming library and sample codes are provided
 * without warranty.
 *
 * We would appreciate it if research outcomes using S2PLOT would
 * provide the following acknowledgement:
 *
 * "Three-dimensional visualisation was conducted with the S2PLOT
 * progamming library"
 *
 * and a reference to
 *
 * D.G.Barnes, C.J.Fluke, P.D.Bourke & O.T.Parry, 2006, Publications
 * of the Astronomical Society of Australia, 23(2), 82-93.
 *
 * S2PLOT development was funded through the Researcher Developer
 * Grant Scheme at Swinburne University of Technology.
 * ==================================================================
 *
 * original version: David Barnes, October 2008
 *
 * $Id: s2plot.js 5329 2008-10-14 04:46:19Z dbarnes $
 *
 */

console.println("* s2plot.js: Adobe 3D JavaScript for S2PLOT-exported 3-d content");
console.println("*");
console.println("* Use of this script implies acceptance of the requirement that at");
console.println("* least one of the following papers be cited when the script is used");
console.println("* to produce a published work:");
console.println("* ");
console.println("* 1. Barnes, D.G., Fluke, C.J., 2008, \"Incorporating interactive");
console.println("*    3-dimensional graphics in astronomy research papers\", New");
console.println("*    Astronomy, 13, 599.");
console.println("*");
console.println("* 2. Fluke, C.J., Barnes, D.G., 2008, \"The Interactive Astronomy");
console.println("*    Textbook, Astronomy Education Review\", 7(1).");
console.println("*");


// arrays of specially-named sections of the model tree
var BBOARDobjsArray = new Array(); // stores the billboard meshes
var BBOARDobjsCount = 0;           // how many billboard meshes are there?
var VRSET1objsArray = new Array();
var VRSET1objsCount = 0;
var VRSET2objsArray = new Array();
var VRSET2objsCount = 0;
var VRSET3objsArray = new Array();
var VRSET3objsCount = 0;
var FRAMEobjsArray = new Array(); // stores the frame meshes
var FRAMEobjsCount = 0;           // how many frame meshes are there?

// temporary variables
var currMesh = null;
var currParentName = "";
var currPPname = "";
var currPPPname = "";
var currPPPPname = "";

// how many individual frames are there?
var nframes = -1;

// compile lists of BBOARD*, VRSET* and FRAME* meshes
for (i=0; i<scene.meshes.count; i++) { 
  currMesh = scene.meshes.getByIndex(i); 
  currParentName = currMesh.parent.name.toString(); 
  currPPname = currMesh.parent.parent.name.toString();
  currPPPname = currMesh.parent.parent.parent.name.toString();
  currPPPPname = currMesh.parent.parent.parent.parent.name.toString();
  framestr = "";
  if (( currParentName.indexOf("BBOARDS1.") > -1 ) ||
      ( currPPname.indexOf("BBOARDS1.") > -1) ||
      ( currPPPname.indexOf("BBOARDS1.") > -1)) {
    BBOARDobjsArray[BBOARDobjsCount] = currMesh;
    BBOARDobjsCount++;
  } else if (( currParentName.indexOf("VRSET1.") > -1) ||
	     ( currPPname.indexOf("VRSET1.") > -1 ) ||
	     ( currPPPname.indexOf("VRSET1.") > -1)) {
    VRSET1objsArray[VRSET1objsCount] = currMesh;
    VRSET1objsCount++;
  } else if (( currParentName.indexOf("VRSET2.") > -1) ||
	     ( currPPname.indexOf("VRSET2.") > -1) ||
	     ( currPPPname.indexOf("VRSET2.") > -1)) {
    VRSET2objsArray[VRSET2objsCount] = currMesh;
    VRSET2objsCount++;
  } else if (( currParentName.indexOf("VRSET3.") > -1) ||
	     ( currPPname.indexOf("VRSET3.") > -1) ||
	     ( currPPPname.indexOf("VRSET3.") > -1)) {
    VRSET3objsArray[VRSET3objsCount] = currMesh;
    VRSET3objsCount++;
  } 

  if ( currParentName.indexOf("FRAME") > -1 ) {
    framestr = currParentName;
  } else if ( currPPname.indexOf("FRAME") > -1 ) {
    framestr = currPPname;
  } else if ( currPPPname.indexOf("FRAME") > -1 ) {
    framestr = currPPPname;
  }
  if (framestr != "") {
    var tmpstr = framestr.match(/FRAME\d+./g)[0];
    var frame = parseInt(tmpstr.match(/\d+/g)[0]);

    if (frame+1 > nframes && frame < 100) {
      nframes = frame+1;
    }

    FRAMEobjsArray[FRAMEobjsCount] = currMesh;
    FRAMEobjsCount++;
  }
 }

// show length of arrays collected
console.println("BBOARDobjsArray.length = " + BBOARDobjsArray.length);
console.println("VRSET1objsArray.length = " + VRSET1objsArray.length);
console.println("VRSET2objsArray.length = " + VRSET2objsArray.length);
console.println("VRSET3objsArray.length = " + VRSET3objsArray.length);
console.println("n frame sets = " + nframes);
console.println("n frame meshes = " + FRAMEobjsCount);

// billboard handling code
function doBBOARDtransform(normal, angle) {
  for (j = 0; j < BBOARDobjsArray.length; j++) {
    var trans = BBOARDobjsArray[j].transform.translation;
    trans.x = -trans.x;
    trans.y = -trans.y;
    trans.z = -trans.z;
    BBOARDobjsArray[j].transform.translateInPlace(trans);

    BBOARDobjsArray[j].transform.rotateAboutVectorInPlace(angle, normal);

    trans.x = -trans.x;
    trans.y = -trans.y;
    trans.z = -trans.z;
    BBOARDobjsArray[j].transform.translateInPlace(trans);
  }
}

function ArbitraryRotate(p, theta, r) {
  var q = new Vector3(0., 0., 0.);
  var costheta = Math.cos(theta);
  var sintheta = Math.sin(theta);
  
  q.x += (costheta + (1 - costheta) * r.x * r.x) * p.x;
  q.x += ((1 - costheta) * r.x * r.y - r.z * sintheta) * p.y;
  q.x += ((1 - costheta) * r.x * r.z + r.y * sintheta) * p.z;
  
  q.y += ((1 - costheta) * r.x * r.y + r.z * sintheta) * p.x;
  q.y += (costheta + (1 - costheta) * r.y * r.y) * p.y;
  q.y += ((1 - costheta) * r.y * r.z - r.x * sintheta) * p.z;
  
  q.z += ((1 - costheta) * r.x * r.z - r.y * sintheta) * p.x;
  q.z += ((1 - costheta) * r.y * r.z + r.x * sintheta) * p.y;
  q.z += (costheta + (1 - costheta) * r.z * r.z) * p.z;
  
  return(q);
}

var last_normal = new Vector3(0., 0., 0.);
var last_angle = 0.;

var last_viewdir = new Vector3(0., 0., 1.);
var last_viewrot = 0.;

function doBillboard() {

  // undo last transform
  doBBOARDtransform(last_normal, -last_angle);

  // calculate new transform via cross and dot product of vd and z
  var camera = scene.cameras.getByIndex(0);
  var camdir = camera.position.subtract(camera.targetPosition);
  camdir.normalize();
  var zax = new Vector3(0., 0., 1.);

  last_normal = camdir.cross(zax);
  last_angle = -Math.acos(camdir.dot(zax));

  var yax = new Vector3(0., 1., 0.);
  // now rotate our yaxis as our billboard has been rotated:
  var rax = ArbitraryRotate(yax, last_angle, last_normal);

  // STILL NEED TO CORRECTLY ORIENT THE BILLBOARDS WITH RESPECT
  // TO THE VIEW DIRECTION.
  // last_viewdir = camdir;
  // last_viewrot = Math.random() * 2 * 3.1415926;
  // last_viewrot = 0.0;

  // do new transformation
  // 1. align normal with view direction
  doBBOARDtransform(last_normal, last_angle);
}

// volume rendering handling code
function setVRSETvis(which, visibility, update) {
  if (which == 0) {
    for (j=0; j<VRSET1objsArray.length; j++) { 
      VRSET1objsArray[j].visible = visibility;
    } 
  } else if (which == 1) {
    for (j=0; j<VRSET2objsArray.length; j++) { 
      VRSET2objsArray[j].visible = visibility;
    }
  } else if (which == 2) {
    for (j=0; j<VRSET3objsArray.length; j++) { 
      VRSET3objsArray[j].visible = visibility;
    }
  }
  if (update) {
    scene.update();
    runtime.refresh();
  }
}

function allSetVis(which) {
  setVRSETvis(which, true, 0);
  setVRSETvis((which + 1) % 3, false, 0);
  setVRSETvis((which + 2) % 3, false, 1);
}

function pickFrameSet() {
  var camera = scene.cameras.getByIndex(0);
  var camdir = camera.position.subtract(camera.targetPosition);
  
  // default
  var whichframeset = 0;
  
  if ((Math.abs(camdir.y) >= Math.abs(camdir.x)) && 
      (Math.abs(camdir.y) >= Math.abs(camdir.z))) {
    whichframeset = 1;
  } else if ((Math.abs(camdir.z) >= Math.abs(camdir.x)) &&
	     (Math.abs(camdir.z) >= Math.abs(camdir.y))) {
    whichframeset = 2;
  }
  allSetVis(whichframeset);
}


mreh = new MouseEventHandler();
mreh.onMouseDown = false;
mreh.onMouseMove = true;
mreh.reportAllTargets = false;
mreh.onEvent = function(event) {
  if (event.leftButtonDown) {
    doBillboard();
    pickFrameSet();
  }
}
runtime.addEventHandler(mreh);

ceh = new RenderEventHandler();
ceh.onEvent = function(event) {
  doBillboard();
  pickFrameSet();
}
runtime.addEventHandler(ceh);

doBillboard();
pickFrameSet();


camspeed = 1.0;
flyCamera = function(amount) {
  // get original position
  var MyCamera = scene.cameras.getByIndex(0);
  originalPos = new Vector3();
  originalPos.set(MyCamera.position);
  // get vector to target
  moveDir = new Vector3();
  moveDir.set(MyCamera.targetPosition);
  moveDir.subtractInPlace(MyCamera.position);
  // modify the position
  MyCamera.position.set(originalPos.addScaled(moveDir, amount * camspeed));
}

debugOut = function() {
  // get and print camera properties
  var MyCamera = scene.cameras.getByIndex(0);

  vp = new Vector3();
  vp.set(MyCamera.position);
  console.println("");
  console.println("vp = " + vp);

  vu = new Vector3();
  vu.set(MyCamera.up);
  vu.subtractInPlace(vp);
  vu.normalize();
  console.println("vu = " + vu);

  vd = new Vector3();
  vd.set(MyCamera.targetPosition);
  vd.subtractInPlace(MyCamera.position);
  vd.normalize();
  console.println("vd = " + vd);

  right = new Vector3();
  right.set(vd.cross(vu));
  right.normalize();
  console.println("right = " + right);

  console.println("camera roll = " + MyCamera.roll);

  console.println("");

  pr = new Vector3();
  pr.set(MyCamera.targetPosition);
}

// rotate (ix,iy) or roll (iz) the camera.
rotateCamera = function(ix, iy, iz) {

  // get camera properties
  var MyCamera = scene.cameras.getByIndex(0);

  vp = new Vector3();
  vp.set(MyCamera.position);

  vu = new Vector3();
  vu.set(MyCamera.up);
  vu.subtractInPlace(vp);
  vu.normalize();

  vd = new Vector3();
  vd.set(MyCamera.targetPosition);
  vd.subtractInPlace(MyCamera.position);
  vd.normalize();

  right = new Vector3();
  right.set(vd.cross(vu));

  pr = new Vector3();
  pr.set(MyCamera.targetPosition);

  delta = camspeed * 3.14159265 / 180.00;
  
  // roll? just change the up vector
  if (Math.abs(iz) > 0.0001) {
    vu.x += iz * right.x * delta;
    vu.y += iz * right.y * delta;
    vu.z += iz * right.z * delta;
    vu.normalize();
    MyCamera.up.set(vu.add(vp));
    return;
  }

  // rotate about focus point, provided camera is not coincident with 
  // this point
  dx = vp.x - pr.x;
  dy = vp.y - pr.y;
  dz = vp.z - pr.z;
  radius = Math.sqrt(dx*dx + dy*dy + dz*dz);
  if (Math.abs(radius) > 0.0001) {
    
    /* Determine the new view point */
    delta *= radius;
    newvp = new Vector3();
    newvp.x = vp.x + delta * ix * right.x + delta * iy * vu.x - pr.x; 
    newvp.y = vp.y + delta * ix * right.y + delta * iy * vu.y - pr.y;
    newvp.z = vp.z + delta * ix * right.z + delta * iy * vu.z - pr.z;
    newvp.normalize();
    vp.x = pr.x + radius * newvp.x;
    vp.y = pr.y + radius * newvp.y;
    vp.z = pr.z + radius * newvp.z;
    
    /* Determine the new right vector */
    newr = new Vector3();
    newr.x = vp.x + right.x - pr.x;
    newr.y = vp.y + right.y - pr.y;
    newr.z = vp.z + right.z - pr.z;
    newr.normalize();
    newr.x = pr.x + radius * newr.x - vp.x;
    newr.y = pr.y + radius * newr.y - vp.y;
    newr.z = pr.z + radius * newr.z - vp.z;
    newr.normalize();
    
    vd.x = pr.x - vp.x;
    vd.y = pr.y - vp.y;
    vd.z = pr.z - vp.z;
    vd.normalize();
    
    /* Determine the new up vector */
    vu = newr.cross(vd);
    vu.normalize();
    
  }

  MyCamera.position.set(vp);
  MyCamera.up.set(vu.add(vp));
}


autospin = 0;
spinspeed = 1;
teh = new TimeEventHandler();
teh.onEvent = function(event) {
  if (autospin) {
    tmp = camspeed;
    camspeed = spinspeed;
    rotateCamera(1., 0., 0.);
    camspeed = tmp;
  }
}
runtime.addEventHandler(teh);


nextFrame = function() {
  if (nframes < 0) {
    return;
  }
  whichframe = (whichframe + 1) % nframes;

  // THIS NEEDS TO BE CHANGED TO USE THE LIST OF FRAME* MESHES
  // AS IT WILL THEN BE A BIT FASTER  
  for (i=0; i<scene.meshes.count; i++) { 
    currMesh = scene.meshes.getByIndex(i); 
    currParentName = currMesh.parent.name.toString(); 
    currPPname = currMesh.parent.parent.name.toString();
    currPPPname = currMesh.parent.parent.parent.name.toString();
    currPPPPname = currMesh.parent.parent.parent.parent.name.toString();
    framestr = "";
    if ( currParentName.indexOf("FRAME") > -1 ) {
      framestr = currParentName;
    } else if ( currPPname.indexOf("FRAME") > -1 ) {
      framestr = currPPname;
    } else if ( currPPPname.indexOf("FRAME") > -1 ) {
      framestr = currPPPname;
    }
    if (framestr != "") {
      var tmpstr = framestr.match(/FRAME\d+./g)[0];
      var frame = parseInt(tmpstr.match(/\d+/g)[0]);
      
      if (frame == whichframe) {
	currMesh.visible = true;
      } else {
	currMesh.visible = false;
      }
    }
  }
  runtime.refresh();
}


myKeyHandler = new KeyEventHandler();
myKeyHandler.onEvent = function( event ) {
  // console.print( "Key pressed with code: " + event.characterCode );
  switch(event.characterCode) {

  case 45: // -,_ : move backwards
  case 95:
    flyCamera(-0.05);
    break;

  case 61: // =,+ : move forwards
  case 43:
    flyCamera(0.05);
    break;

  case 65: // A,a : toggle autospin
  case 97: 
    autospin = (autospin + 1) % 2;
    break;

  case 68: // D,d : print debug info
  case 100:
    debugOut();
    break;

  case 62: // > : increase camera delta / step
    camspeed = camspeed * 2.5;
    break;
  case 60: // < : decrease camera delta / step
    camspeed = camspeed * 0.4;
    break;

  case 42: // * : increase spin speed
    spinspeed = spinspeed * 2.5;
    break;
  case 47: // / : decrease spin speed
    spinspeed = spinspeed * 0.4;
    break;

  case 28: // left-arrow : rotate *camera* left
    rotateCamera(-1., 0., 0.);
    break;
  case 29: // right-arrow : rotate *camera* right
    rotateCamera(1., 0., 0.);
    break;
  case 30: // up-arrow : rotate up and over
    rotateCamera(0., 1., 0.);
    break;
  case 31: // down-arrow : rotate down and under
    rotateCamera(0., -1., 0.);
    break;
    
  case 91: // [ : roll ccw
    rotateCamera(0., 0., -1.);
    break;
  case 93: // ] : roll cw
    rotateCamera(0., 0., 1.);
    break;
    
  case 32: // space : next frame
    nextFrame();
    break;

  }
}
runtime.addEventHandler( myKeyHandler );

if (nframes > 0) {
  whichframe = -1;
  nextFrame();
}
